#include "testlib.h"
#include <iostream>
#include <string>
#include <vector>
#include <assert.h>
#include <bitset>

#define rep(i, l, r) for (int i = l; i < r; i++)
#define repb(i, l, r) for (int i = l; i > r; i--)
#define sz(a) (int)a.size()

using namespace std;

string shuffleString(string s, vector<int> p) {
    int n = sz(s);
    string res(n, ' ');
    rep(i, 0, n) {
        res[p[i]] = s[i];
    }
    return res;
}

void shuffleTest(string &s, vector<string> &tests) {
    int n = sz(s);
    vector<int> p(n);
    rep(i, 0, n) {
        p[i] = i;
    }
    shuffle(p.begin(), p.end());
    s = shuffleString(s, p);
    rep(i, 0, sz(tests)) {
        tests[i] = shuffleString(tests[i], p);
    }
    shuffle(tests.begin(), tests.end());
}

vector<int> getPartition(int m, int blocks) {
    set<int> s;
    s.insert(0);
    s.insert(m);
    while (sz(s) < blocks + 1) {
        s.insert(rnd.next(0, m));
    }
    vector<int> res;
    int last = -1;
    for (int it : s) {
        if (last != -1) {
            res.push_back(it - last);
        }
        last = it;
    }
    return res;
}

void createBlock(int n1, int n2, int k1, int k2, int m, string &answer, vector<string> &tests) {
    string wrong(n1 + n2, ' ');
    rep(i, 0, n1 + n2) {
        while (true) {
            wrong[i] = char('A' + rnd.next(0, 3));
            if (wrong[i] != answer[i]) {
                break;
            }
        }
    }

    vector<int> pos1(n1 + n2);
    rep(i, 0, n1 + n2) {
        pos1[i] = i;
    }
    shuffle(pos1.begin(), pos1.end());
    vector<int> pos2;
    rep(i, 0, n2) {
        pos2.push_back(pos1.back());
        pos1.pop_back();
    }
    rep(i, 0, m) {
        string test(n1 + n2, ' ');
        for (int p : pos1) {
            test[p] = answer[p];
        }
        for (int p : pos2) {
            test[p] = wrong[p];
        }
        rep(j, 0, k1) {
            int p = pos1[rnd.next(0, sz(pos1) - 1)];
            test[p] = char('A' + rnd.next(0, 3));
        }
        rep(j, 0, k2) {
            int p = pos2[rnd.next(0, sz(pos2) - 1)];
            test[p] = char('A' + rnd.next(0, 3));
        }
        tests.push_back(test);
    }
}

int main(int argc, char* argv[])
{
    registerGen(argc, argv, 1);

    int n1 = atoi(argv[1]);
    int n2 = atoi(argv[2]);
    int k1 = atoi(argv[3]);
    int k2 = atoi(argv[4]);
    int m = atoi(argv[5]);
    int blocks = atoi(argv[6]);

    string answer(n1 + n2, ' ');
    rep(i, 0, n1 + n2) {
        answer[i] = char('A' + rnd.next(0, 3));
    }

    vector<string> tests;
    vector<int> partition = getPartition(m, blocks);

    for (int cnt : partition) {
        createBlock(n1, n2, k1, k2, cnt, answer, tests);
    }

    shuffleTest(answer, tests);

    cout << n1 + n2 << "\n";
    cout << answer << "\n";
    cout << m << "\n";
    for (auto it : tests) {
        cout << it << "\n";
    }

    return 0;
}